//-------------------------------------------------------------------------
//
//  This is part of the Microsoft Tablet PC Platform SDK
//  Copyright (C) 2002 Microsoft Corporation
//  All rights reserved.
//
//  This source code is only intended as a supplement to the
//  Microsoft Tablet PC Platform SDK Reference and related electronic 
//  documentation provided with the Software Development Kit.
//  See these sources for more detailed information. 
//
//  File: InkCollection.cs
//  Simple Ink Collection Sample Application
//
//  This sample program is a "Hello World" application, which
//  demonstrates basic functionality of the Tablet PC platform. 
//  It is the simplest program that you can build using the 
//  Tablet PC Platform APIs. This application uses an ink
//  collector to collect and render pen input. 
//
//  The features used are: InkCollector, default tablet, and 
//  modifying default drawing attributes.
//
//--------------------------------------------------------------------------

using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Diagnostics;

// The Ink namespace, which contains the Tablet PC Platform API
using Microsoft.Ink;


namespace InkCollection
{
    /// <summary>
    /// The InkCollection Sample Application form class
    /// </summary>
    public class InkCollection : System.Windows.Forms.Form
    {
		public const String ptDirPath = ""; //Path to PostTrain's Dir
		public const String ptPath = ptDirPath + "PostTrain.exe"; //Path to PostTrain itself
		public const String dataPath = ptDirPath + "data\\"; //Path to training data

        // Declare the Ink Collector object
        private InkCollector myInkCollector = null;

        // Declare constants for the pen widths used by this application.
        // Note that these constants are in high metric units (1 unit = .01mm)
        private const float ThinInkWidth = 10;
        private const float MediumInkWidth = 100;
        private const float ThickInkWidth = 200;

		private int tcount = 0;

        #region Standard Template Code

        private System.Windows.Forms.MenuItem miMainFile;
        private System.Windows.Forms.MenuItem miExit;
        private System.Windows.Forms.MenuItem miMainInk;
        private System.Windows.Forms.MenuItem miEnabled;
        private System.Windows.Forms.MenuItem miColor;
        private System.Windows.Forms.MenuItem miRed;
        private System.Windows.Forms.MenuItem miGreen;
        private System.Windows.Forms.MenuItem miBlue;
        private System.Windows.Forms.MenuItem miBlack;
        private System.Windows.Forms.MenuItem miWidth;
        private System.Windows.Forms.MenuItem miThin;
        private System.Windows.Forms.MenuItem miMedium;
        private System.Windows.Forms.MenuItem miThick;
        private System.Windows.Forms.MainMenu miMain;
		private System.Windows.Forms.MenuItem miPurple;
		private System.Windows.Forms.MenuItem miCurveFit;
		private System.Windows.Forms.MenuItem miOptions;
		private System.Windows.Forms.MenuItem miClear;
		private System.Windows.Forms.MenuItem miGetInk;
		private System.Windows.Forms.MenuItem miRecognize;
		private System.Windows.Forms.MenuItem miGetCoords;
		private System.Windows.Forms.MenuItem miDllTest;
		private System.Windows.Forms.MenuItem miTrain;
		private System.Windows.Forms.MenuItem miTrainAdd;
		private System.Windows.Forms.MenuItem miTrainClear;
		private System.Windows.Forms.MenuItem miTrainSave;
		private System.Windows.Forms.MenuItem miPostTrain;
		private System.Windows.Forms.MenuItem miReloadRec;
		private System.Windows.Forms.Button bAddPres;
		private System.Windows.Forms.Button bRecognize;
		private System.Windows.Forms.Button bClear;
		private System.Windows.Forms.TextBox tbAlts;
		private System.Windows.Forms.Button bClearTraining;
		private System.Windows.Forms.Button bSaveTraining;
        private System.ComponentModel.Container components = null;
        #endregion

        public InkCollection()
        {
            #region Standard Template Code
            // Required for Windows Form Designer support
            InitializeComponent();
            #endregion
        }

        #region Standard Template Code
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if (components != null) 
                {
                    components.Dispose();
                }
                if (myInkCollector != null)
                {
                    myInkCollector.Dispose();
                }
            }
            base.Dispose( disposing );
        }
        #endregion


		#region Windows Form Designer generated code
        /// <summary>
        /// ----- Standard Template Code -----
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
			this.miMain = new System.Windows.Forms.MainMenu();
			this.miMainFile = new System.Windows.Forms.MenuItem();
			this.miExit = new System.Windows.Forms.MenuItem();
			this.miMainInk = new System.Windows.Forms.MenuItem();
			this.miEnabled = new System.Windows.Forms.MenuItem();
			this.miColor = new System.Windows.Forms.MenuItem();
			this.miRed = new System.Windows.Forms.MenuItem();
			this.miGreen = new System.Windows.Forms.MenuItem();
			this.miBlue = new System.Windows.Forms.MenuItem();
			this.miBlack = new System.Windows.Forms.MenuItem();
			this.miPurple = new System.Windows.Forms.MenuItem();
			this.miWidth = new System.Windows.Forms.MenuItem();
			this.miThin = new System.Windows.Forms.MenuItem();
			this.miMedium = new System.Windows.Forms.MenuItem();
			this.miThick = new System.Windows.Forms.MenuItem();
			this.miCurveFit = new System.Windows.Forms.MenuItem();
			this.miOptions = new System.Windows.Forms.MenuItem();
			this.miClear = new System.Windows.Forms.MenuItem();
			this.miGetInk = new System.Windows.Forms.MenuItem();
			this.miRecognize = new System.Windows.Forms.MenuItem();
			this.miGetCoords = new System.Windows.Forms.MenuItem();
			this.miDllTest = new System.Windows.Forms.MenuItem();
			this.miReloadRec = new System.Windows.Forms.MenuItem();
			this.miTrain = new System.Windows.Forms.MenuItem();
			this.miTrainAdd = new System.Windows.Forms.MenuItem();
			this.miTrainSave = new System.Windows.Forms.MenuItem();
			this.miPostTrain = new System.Windows.Forms.MenuItem();
			this.miTrainClear = new System.Windows.Forms.MenuItem();
			this.bAddPres = new System.Windows.Forms.Button();
			this.bRecognize = new System.Windows.Forms.Button();
			this.bClear = new System.Windows.Forms.Button();
			this.tbAlts = new System.Windows.Forms.TextBox();
			this.bClearTraining = new System.Windows.Forms.Button();
			this.bSaveTraining = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// miMain
			// 
			this.miMain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				   this.miMainFile,
																				   this.miMainInk,
																				   this.miOptions,
																				   this.miTrain});
			// 
			// miMainFile
			// 
			this.miMainFile.Index = 0;
			this.miMainFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					   this.miExit});
			this.miMainFile.Text = "File";
			// 
			// miExit
			// 
			this.miExit.Index = 0;
			this.miExit.Text = "Exit";
			this.miExit.Click += new System.EventHandler(this.miExit_Click);
			// 
			// miMainInk
			// 
			this.miMainInk.Index = 1;
			this.miMainInk.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.miEnabled,
																					  this.miColor,
																					  this.miWidth,
																					  this.miCurveFit});
			this.miMainInk.Text = "Ink";
			// 
			// miEnabled
			// 
			this.miEnabled.Checked = true;
			this.miEnabled.Index = 0;
			this.miEnabled.Text = "Enabled";
			this.miEnabled.Click += new System.EventHandler(this.miEnabled_Click);
			// 
			// miColor
			// 
			this.miColor.Index = 1;
			this.miColor.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.miRed,
																					this.miGreen,
																					this.miBlue,
																					this.miBlack,
																					this.miPurple});
			this.miColor.Text = "Color";
			// 
			// miRed
			// 
			this.miRed.Index = 0;
			this.miRed.Text = "Red";
			this.miRed.Click += new System.EventHandler(this.miRed_Click);
			// 
			// miGreen
			// 
			this.miGreen.Index = 1;
			this.miGreen.Text = "Green";
			this.miGreen.Click += new System.EventHandler(this.miGreen_Click);
			// 
			// miBlue
			// 
			this.miBlue.Index = 2;
			this.miBlue.Text = "Blue";
			this.miBlue.Click += new System.EventHandler(this.miBlue_Click);
			// 
			// miBlack
			// 
			this.miBlack.Index = 3;
			this.miBlack.RadioCheck = true;
			this.miBlack.Text = "Black";
			this.miBlack.Click += new System.EventHandler(this.miBlack_Click);
			// 
			// miPurple
			// 
			this.miPurple.Index = 4;
			this.miPurple.Text = "Purple";
			this.miPurple.Click += new System.EventHandler(this.miPurple_Click);
			// 
			// miWidth
			// 
			this.miWidth.Index = 2;
			this.miWidth.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.miThin,
																					this.miMedium,
																					this.miThick});
			this.miWidth.Text = "Width";
			// 
			// miThin
			// 
			this.miThin.Index = 0;
			this.miThin.Text = "Thin";
			this.miThin.Click += new System.EventHandler(this.miThin_Click);
			// 
			// miMedium
			// 
			this.miMedium.Index = 1;
			this.miMedium.Text = "Medium";
			this.miMedium.Click += new System.EventHandler(this.miMedium_Click);
			// 
			// miThick
			// 
			this.miThick.Index = 2;
			this.miThick.Text = "Thick";
			this.miThick.Click += new System.EventHandler(this.miThick_Click);
			// 
			// miCurveFit
			// 
			this.miCurveFit.Checked = true;
			this.miCurveFit.Index = 3;
			this.miCurveFit.Text = "Curve Fit";
			this.miCurveFit.Click += new System.EventHandler(this.miCurve_Click);
			// 
			// miOptions
			// 
			this.miOptions.Index = 2;
			this.miOptions.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.miClear,
																					  this.miGetInk,
																					  this.miRecognize,
																					  this.miGetCoords,
																					  this.miDllTest,
																					  this.miReloadRec});
			this.miOptions.Text = "Options";
			// 
			// miClear
			// 
			this.miClear.Index = 0;
			this.miClear.Text = "Clear";
			this.miClear.Click += new System.EventHandler(this.miClear_Click);
			// 
			// miGetInk
			// 
			this.miGetInk.Index = 1;
			this.miGetInk.Text = "Get Ink";
			this.miGetInk.Click += new System.EventHandler(this.miGetInk_Click);
			// 
			// miRecognize
			// 
			this.miRecognize.Index = 2;
			this.miRecognize.Text = "Recognize";
			this.miRecognize.Click += new System.EventHandler(this.miRecognize_Click);
			// 
			// miGetCoords
			// 
			this.miGetCoords.Index = 3;
			this.miGetCoords.Text = "Get Coords";
			this.miGetCoords.Click += new System.EventHandler(this.miGetCoord_Click);
			// 
			// miDllTest
			// 
			this.miDllTest.Index = 4;
			this.miDllTest.Text = "Dll Test";
			this.miDllTest.Click += new System.EventHandler(this.miDllTest_Click);
			// 
			// miReloadRec
			// 
			this.miReloadRec.Index = 5;
			this.miReloadRec.Text = "Reload Recognizer";
			this.miReloadRec.Click += new System.EventHandler(this.miReloadRec_Click);
			// 
			// miTrain
			// 
			this.miTrain.Index = 3;
			this.miTrain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.miTrainAdd,
																					this.miTrainSave,
																					this.miPostTrain,
																					this.miTrainClear});
			this.miTrain.Text = "Training";
			// 
			// miTrainAdd
			// 
			this.miTrainAdd.Index = 0;
			this.miTrainAdd.Text = "Add Present";
			this.miTrainAdd.Click += new System.EventHandler(this.miTrainEntry_Click);
			// 
			// miTrainSave
			// 
			this.miTrainSave.Index = 1;
			this.miTrainSave.Text = "Save Training Data";
			this.miTrainSave.Click += new System.EventHandler(this.miSaveTraining_Click);
			// 
			// miPostTrain
			// 
			this.miPostTrain.Index = 2;
			this.miPostTrain.Text = "Run PostTrain";
			this.miPostTrain.Click += new System.EventHandler(this.miPostTrain_Click);
			// 
			// miTrainClear
			// 
			this.miTrainClear.Index = 3;
			this.miTrainClear.Text = "Clear All Training";
			this.miTrainClear.Click += new System.EventHandler(this.miClearTraining_Click);
			// 
			// bAddPres
			// 
			this.bAddPres.Location = new System.Drawing.Point(16, 312);
			this.bAddPres.Name = "bAddPres";
			this.bAddPres.TabIndex = 0;
			this.bAddPres.Text = "Train Entry";
			this.bAddPres.Click += new System.EventHandler(this.miTrainEntry_Click);
			// 
			// bRecognize
			// 
			this.bRecognize.Location = new System.Drawing.Point(216, 312);
			this.bRecognize.Name = "bRecognize";
			this.bRecognize.TabIndex = 1;
			this.bRecognize.Text = "Recognize";
			this.bRecognize.Click += new System.EventHandler(this.miRecognize_Click);
			// 
			// bClear
			// 
			this.bClear.Location = new System.Drawing.Point(104, 312);
			this.bClear.Name = "bClear";
			this.bClear.Size = new System.Drawing.Size(96, 23);
			this.bClear.TabIndex = 2;
			this.bClear.Text = "Clear Screen";
			this.bClear.Click += new System.EventHandler(this.miClear_Click);
			// 
			// tbAlts
			// 
			this.tbAlts.Location = new System.Drawing.Point(304, 8);
			this.tbAlts.Multiline = true;
			this.tbAlts.Name = "tbAlts";
			this.tbAlts.ReadOnly = true;
			this.tbAlts.Size = new System.Drawing.Size(176, 264);
			this.tbAlts.TabIndex = 3;
			this.tbAlts.Text = "";
			// 
			// bClearTraining
			// 
			this.bClearTraining.Location = new System.Drawing.Point(304, 280);
			this.bClearTraining.Name = "bClearTraining";
			this.bClearTraining.Size = new System.Drawing.Size(176, 24);
			this.bClearTraining.TabIndex = 4;
			this.bClearTraining.Text = "Clear All Training Data";
			this.bClearTraining.Click += new System.EventHandler(this.miClearTraining_Click);
			// 
			// bSaveTraining
			// 
			this.bSaveTraining.Location = new System.Drawing.Point(304, 312);
			this.bSaveTraining.Name = "bSaveTraining";
			this.bSaveTraining.Size = new System.Drawing.Size(176, 24);
			this.bSaveTraining.TabIndex = 5;
			this.bSaveTraining.Text = "Save And Reload Training Data";
			this.bSaveTraining.Click += new System.EventHandler(this.miSaveTraining_Click);
			// 
			// InkCollection
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(494, 347);
			this.Controls.Add(this.bSaveTraining);
			this.Controls.Add(this.bClearTraining);
			this.Controls.Add(this.tbAlts);
			this.Controls.Add(this.bClear);
			this.Controls.Add(this.bRecognize);
			this.Controls.Add(this.bAddPres);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Menu = this.miMain;
			this.MinimizeBox = false;
			this.Name = "InkCollection";
			this.Text = "CIT Ink";
			this.Load += new System.EventHandler(this.InkCollection_Load);
			this.ResumeLayout(false);

		}
        #endregion

        #region Standard Template Code
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main() 
        {
            Application.Run(new InkCollection());
        }
        #endregion      

        /// <summary>
        /// Event Handler from Form Load Event
        /// Setup the ink collector for collection
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void InkCollection_Load(object sender, System.EventArgs e)
        {

            // Create a new ink collector and assign it to this form's window
            myInkCollector = new InkCollector(this.Handle);

            // Set the pen width to be a medium width
            myInkCollector.DefaultDrawingAttributes.Width = MediumInkWidth;
			myInkCollector.DefaultDrawingAttributes.FitToCurve = true;
            // 
            // If you do not modify the default drawing attributes, the default 
            // drawing attributes will use the following properties and values:
            // 
            //      Color           = black
            //      Width           = 53 (2 pixels on a 96 dpi screen)
            //      Height          = 1
            //      PenStyle        = solid, geometric
            //      PenTip          = ball
            //      Transparency    = 0
            //      AntiAliased     = true
            //      FitToCurve      = false
            // 
            // For an example of how to modify other drawing attributes, uncomment
            // the following lines of code:
            // myInkCollector.DefaultDrawingAttributes.PenTip = PenTip.Rectangle;
            // myInkCollector.DefaultDrawingAttributes.Height = (.5F)*MediumInkWidth;
            // myInkCollector.DefaultDrawingAttributes.Transparency = 128;
            //

            // Turn the ink collector on
            myInkCollector.Enabled = true;        
			myInkCollector.Stroke += new InkCollectorStrokeEventHandler(icStrokeHandler);
			miReloadRec_Click(sender, e);
        }

		private void icStrokeHandler(object sender, InkCollectorStrokeEventArgs e)
		{
			miRecognize_Click(sender, e);
		}

        /// <summary>
        /// Event Handler from Ink->Color->Red Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miRed_Click(object sender, System.EventArgs e)
        {
            myInkCollector.DefaultDrawingAttributes.Color = Color.Red;
        }

        /// <summary>
        /// Event Handler from Ink->Color->Green Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miGreen_Click(object sender, System.EventArgs e)
        {
            myInkCollector.DefaultDrawingAttributes.Color = Color.Green;
        }

        /// <summary>
        /// Event Handler from Ink->Color->Blue Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miBlue_Click(object sender, System.EventArgs e)
        {
            myInkCollector.DefaultDrawingAttributes.Color = Color.Blue;
        }

        /// <summary>
        /// Event Handler from Ink->Color->Black Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miBlack_Click(object sender, System.EventArgs e)
        {
            myInkCollector.DefaultDrawingAttributes.Color = Color.Black;
        }
	
		private void miPurple_Click(object sender, System.EventArgs e)
		{

			myInkCollector.DefaultDrawingAttributes.Color = Color.Purple;
		}
        /// <summary>
        /// Event Handler from Ink->Width->Thin Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miThin_Click(object sender, System.EventArgs e)
        {
            myInkCollector.DefaultDrawingAttributes.Width = ThinInkWidth;
        }

        /// <summary>
        /// Event Handler from Ink->Width->Medium Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miMedium_Click(object sender, System.EventArgs e)
        {
            myInkCollector.DefaultDrawingAttributes.Width = MediumInkWidth;
        }
        /// <summary>
        /// Event Handler from Ink->Width->Thick Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miThick_Click(object sender, System.EventArgs e)
        {
            myInkCollector.DefaultDrawingAttributes.Width = ThickInkWidth;
        }

        /// <summary>
        /// Event Handler from File->Exit Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miExit_Click(object sender, System.EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Event Handler from Ink->Enabled Menu Item
        /// </summary>
        /// <param name="sender">The control that raised the event.</param>
        /// <param name="e">The event arguments.</param>
        private void miEnabled_Click(object sender, System.EventArgs e)
        {
            // Toggle the enabled state of the ink collector
            myInkCollector.Enabled = !myInkCollector.Enabled;
            miEnabled.Checked = myInkCollector.Enabled;
        }

		private void miCurve_Click(object sender, System.EventArgs e)
		{

			myInkCollector.DefaultDrawingAttributes.FitToCurve = !myInkCollector.DefaultDrawingAttributes.FitToCurve;
			miCurveFit.Checked = myInkCollector.DefaultDrawingAttributes.FitToCurve;
		}

		private void miClear_Click(object sender, System.EventArgs e)
		{
			myInkCollector.Ink.DeleteStrokes();
			this.Invalidate();
			this.Update();
		}
		
		private void miGetInk_Click(object sender, System.EventArgs e)
		{			

			FileStream fso;
			byte[] myInk;
			
			myInk= myInkCollector.Ink.Save(PersistenceFormat.Gif,
					CompressionMode.NoCompression);
			fso = new FileStream("out.gif", FileMode.Create);
			BinaryWriter w = new BinaryWriter(fso);

			w.Write(myInk);
			w.Flush();
			fso.Flush();
			fso.Close();
		}

		private void miRecognize_Click(object sender, System.EventArgs e)
		{
			Strokes daStrokes = myInkCollector.Ink.Strokes;
			char []buf = new char[128];
			float prob;
			int[] xs; int[] ys;
			String []alts = new String[10];
			//String bestString ="";

			xs = xCoordsFromStrokes(daStrokes);
			ys = yCoordsFromStrokes(daStrokes);
			
			LoadProfWithRec(xs, ys, xs.Length-1);
			for(int i = 0; i < 10; i++)
			{
				prob = ProfProbOfChoice(i);
				if(0==ProfNameOfChoice(i, buf))
				{
					buf[buf.Length-1] = '\n';
					String bufVal = new String(buf, 0, CStringLength(buf));
					alts[i] = i + ") " +bufVal + "\tWith Prob = " + prob.ToString();
					//bestString += prob.ToString() + " ID = " + bufVal+"\n";
									
				}
			}
			tbAlts.Lines = alts;
			//tbAlts.Text=bestString;
			//MessageBox.Show(bestString);
		}

		private void miGetCoord_Click(object sender, System.EventArgs e)
		{
			Strokes daStrokes;
			Stroke presStroke;
			int i,j;
			Point[] presPoints;
			string strokesStr="Strokes are as follows:";

			daStrokes = myInkCollector.Ink.Strokes;

			for(i=0; i < daStrokes.Count; i++)
			{
				presStroke = daStrokes[i];
				presPoints = presStroke.GetPoints();
				strokesStr += "S#" + i + ", " + presPoints.Length+ " points:" ;
				myInkCollector.Renderer.InkSpaceToPixel(this.Handle, ref presPoints);

				for(j=0; j < presPoints.Length ; j++)
					strokesStr += "(" + presPoints[j].X + "," +presPoints[j].Y +") ";
				strokesStr+="\n";
			}

			MessageBox.Show(strokesStr);
		}

		private void miReloadRec_Click(object sender, System.EventArgs e)
		{
			LoadAsRecognizer(dataPath.ToCharArray());
		}

		private void miDllTest_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show("For input = 2 Test Square outputs: " + TestSquare(2));
		}

		private void miTrainEntry_Click(object sender, System.EventArgs e)
		{
			Strokes daStrokes = myInkCollector.Ink.Strokes;
			int[] xs; int[] ys;

			xs = xCoordsFromStrokes(daStrokes);
			ys = yCoordsFromStrokes(daStrokes);
		
			if(xs.Length == 0)
			{
				tbAlts.Text = "Cannot Add empty screen to training data";
				return;
			}

			AddToTraining(xs, ys, xs.Length);
			
			myInkCollector.Ink.DeleteStrokes();
			
			tbAlts.Text = "Just Added Training Entry #:" + tcount;
			tcount ++;

			this.Invalidate();
			this.Update();
		}

		private void miClearTraining_Click(object sender, System.EventArgs e)
		{
			ClearTraining();
			myInkCollector.Ink.DeleteStrokes();
			this.Invalidate();
			this.Update();
			tbAlts.Text = "Just Reset Training, now no entries";
			tcount = 0;
		}

		private void miSaveTraining_Click(object sender, System.EventArgs e)
		{
			SaveDialog aSD = new SaveDialog();
			aSD.ShowDialog();
			
			if(!aSD.lastClickWasCancel())
			{
				miClearTraining_Click(sender,e);
				miPostTrain_Click(sender, e);
			}
		}

		private void miPostTrain_Click(object sender, System.EventArgs e)
		{
			Process aProcess;
			ProcessStartInfo psi = new ProcessStartInfo(ptPath);
			
			psi.WorkingDirectory = ptDirPath;
			psi.WindowStyle = ProcessWindowStyle.Hidden;
			aProcess = Process.Start(psi);
			aProcess.WaitForExit(10000);
			miReloadRec_Click(sender, e);
		}

		private int[] xCoordsFromStrokes(Strokes daStrokes)
		{
			int[] returnme;
			int i, j, totalpoints=0, presindex=0;
			Stroke presStroke;
			Point[] presPoints;
			
			for(i=0; i < daStrokes.Count; i++)
				totalpoints += daStrokes[i].GetPoints().Length + 1;

			returnme = new int[totalpoints];
 
			for(i=0; i < daStrokes.Count; i++)
			{
				presStroke = daStrokes[i];
				presPoints = presStroke.GetPoints();
				myInkCollector.Renderer.InkSpaceToPixel(this.Handle, ref presPoints);
				for(j=0; j < presPoints.Length; j+=2)
				{
					returnme[presindex] = presPoints[j].X; //May need to change this constant
					presindex++;
				}
				returnme[presindex] = 0;
				presindex ++;
			}

			return returnme;
		}

		private int[] yCoordsFromStrokes(Strokes daStrokes)
		{
			int[] returnme;
			int i, j, totalpoints=0, presindex=0;
			Stroke presStroke;
			Point[] presPoints;
			
			for(i=0; i < daStrokes.Count; i++)
				totalpoints += daStrokes[i].GetPoints().Length + 1;

			returnme = new int[totalpoints];
 
			for(i=0; i < daStrokes.Count; i++)
			{
				presStroke = daStrokes[i];
				presPoints = presStroke.GetPoints();
				myInkCollector.Renderer.InkSpaceToPixel(this.Handle, ref presPoints);
				for(j=0; j < presPoints.Length; j+=2)
				{
					returnme[presindex] = presPoints[j].Y; //May need to change this constant
					presindex++;
				}
				returnme[presindex] = 0;
				presindex ++;
			}

			return returnme;
		}

		private static int CStringLength(char []aCstr)
		{
			int i;

			for(i=0; i < aCstr.Length; i++)
				if(aCstr[i] == '\0')
					return i;
			return i;
		}

		[DllImport("WinCitDll.dll")]
		public static extern int TestSquare(int input);
		[DllImport("WinCitDll.dll")]
		public static extern float ProbOfChoice(int choicenum, [In] int[] xs, [In] int[] ys, int arrsize, [In] char[] path);
		[DllImport("WinCitDll.dll")]
		public static extern int NameOfChoice(int choicenum, [In] int[] xs, [In] int[] ys, int arrsize, [Out] char[] buf, [In] char[] path);
		[DllImport("WinCitDll.dll")]
		public static extern float ProbOfBest([In] int []xs, [In] int[] ys, int arrsize, [In] char[] path);
		[DllImport("WinCitDll.dll")]
		public static extern int NameOfBest([In] int[] xs, [In] int[] ys, int arrsize, [Out] char[] buf, [In] char []path);
		[DllImport("WinCitDll.dll")]
		public static extern int LoadAsProfile([In] int[] xs, [In] int[] ys, int arrsize, char []path);
		[DllImport("WinCitDll.dll")]
		public static extern float ProfProbOfChoice(int choicenum);
		[DllImport("WinCitDll.dll")]
		public static extern int ProfNameOfChoice(int choicenum, [Out] char []buf);
		[DllImport("WinCitDll.dll")]
		public static extern void AddToTraining([In] int []xs, [In] int []ys, int arrsize);
		[DllImport("WinCitDll.dll")]
		public static extern void ClearTraining();
		[DllImport("WinCitDll.dll")]
		public static extern void LoadAsRecognizer([In] char []path);
		[DllImport("WinCitDll.dll")]
		public static extern void LoadProfWithRec([In] int []xs, [In] int []ys, int arrsize);		
	}
}
