/***************************************************************************
* Trans2.C																   *
*																		   *
* Two-dimensional transformations with uniform (isotropic) scaling. 	   *
*																		   *
*	Changes:															   *
*																		   *
*		07/04/98	arvo	First working version.						   *
*		04/30/98	arvo	Initial coding. 							   *
*																		   *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo										   *
*																		   *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html	   *
*																		   *
* This program is distributed in the hope that it will be useful, but	   *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more	   *
* details.																   *
*																		   *
***************************************************************************/
#include <stdio.h>
#include <fstream>
using namespace std;
#include "../util/Char.h"
#include "../util/Trans2.h"

const Trans2 Trans2::Null ( Vec2( 0, 0 ), 0 );
const Trans2 Trans2::Ident( Vec2( 0, 0 ), 1 );

// Most of the functions are currently inline.

ostream &operator<<( ostream &out, const Trans2 &T )
{
	out << "Trans2: (" << T.X() << ", " << T.Y() << ") " << T.Scale() << endl << endl;
	return out;
}

const char *Trans2::ToString( char *buff ) const
{
	static char local_buff[64];
	if( buff == NULL ) buff = local_buff;
	sprintf( buff, "%lX %lX %lX", 
		float_as_long( X() ), 
		float_as_long( Y() ), 
		float_as_long( Scale() )
		);
	return buff;
}

Trans2 &Trans2::FromString( const char *str )
{
	long long_x, long_y, long_s;
	sscanf( str, "%lX %lX %lX", &long_x, &long_y, &long_s );
	X()   = long_as_float( long_x );
	Y()   = long_as_float( long_y );
	scale = long_as_float( long_s );
	return *this;
}

Trans2 Trans2::Scale( double scale )
{ 
	return Trans2( Vec2::Zero, scale ); 
}

double dist( const Box2d &box, const Trans2 &A, const Trans2 &B )
{
	Box2d box_A( A * box );
	Box2d box_B( B * box );
	double d1 = dist( box_A.NE(), box_B.NE() );
	double d2 = dist( box_A.NW(), box_B.NW() );
	double d3 = dist( box_A.SE(), box_B.SE() );
	double d4 = dist( box_A.SW(), box_B.SW() );
	return Max( Max( d1, d2 ), Max( d3, d4 ) );
}

Trans2 Box_to_Box( const Box2d &from, const Box2d &to, int match_x )
{
	double scale;
	double dx = 0;
	double dy = 0;
	Vec2   origin;
	if( match_x )
	{
		// Create a transform that maps the X range of one box to another.	
		// Center Y range within the Y range of the other.
		scale  = to.Width() / from.Width();
		dy	   = to.Height() - scale * from.Height();
		origin = to.Min()	 - scale * from.Min();
	}
	else
	{
		// Create a transform that maps the Y range of one box to another.	
		// Center X range within the X range of the other.
		scale  = to.Height() / from.Height();
		dx	   = to.Width() - scale * from.Width();
		origin = to.Min()	- scale * from.Min();
	}
	
	// Finally, add in half the difference in the coordinate that
	// does not exactly match.
	origin += 0.5 * Vec2( dx, dy );
	
	return Trans2( origin, scale );
}

#include "stdafx.h"
